%This script is designed to solve the position of a hydraulic jump
%downstream from a gate, with a downstream water surface, or critical flow
%at the end of the channel.  It uses the first method described in the
%book, e.g. solve the two GVF profiles from upstream and downstream and
% then interpolating the position where the two momentum functions equal.
%It performs the same tasks as the programs listed in the text with names
%GVFJMP. (This script and gvfjmp2 both use fdyxjmp to supply the derivative
%dY/dx to the ODE solver, or the built-in MATLAB "ode45.") The dialog in
%the command window to solve Example Problem 12 with a downstream water 
%surface elevation, WS, of 4.9 ft is as follows:
%>>gvfjmp
%Give: Q,b,m,So,n,L,WS,Yb,g within [ ] [480 8 0 .0011 .013 800 4.9 2 32.2]
%Critical Depth =    4.8174
%Upsteam GVF-profile
%      x        Y       Mom.     Fr
%    0.00     2.000     463.20    3.74
%   40.00     2.143     435.78    3.37
%   . 
%  320.00     3.287     315.34    1.77
%  360.00     3.491     304.93    1.62
%  400.00     3.721     295.77    1.47
%   . 
%  800.00     5.086     254.00    0.77
%Downsteam GVF-profile
%      x        Y       Mom.      Fr
%  800.00     4.900     278.57    0.97
%  760.00     5.387     282.11    0.85
%   .
%  360.00     6.479     305.94    0.64
%  320.00     6.535     307.69    0.63
%  280.00     6.588     309.38    0.63
% 
%   40.00     6.850     318.25    0.59
%    0.00     6.886     319.57    0.59
%Position of Hydraulic jump =    355.34 & Momentum=     306.15
%Depth upstream =     3.468, Depth downstream =     6.485

global b m So L Yb FNQ QG2 m2 fm2 Cu WS
var=input('Give: Q,b,m,So,n,L,WS,Yb,g within [ ] ');
Q=var(1);b=var(2);m=var(3);So=var(4);n=var(5);L=var(6);WS=var(7);Yb=var(8);g=var(9);
Cu=1;
if g>20, Cu=1.486; end;  QG2=Q^2/g;  FNQ=n*Q/Cu; m2=2*m; fm2=2*sqrt(m^2+1);
if m<1.e-5
    Yc=((Q/b)^2/g)^.33333333;
else
    Yc=.925*b*(.5*(m^3*Q^2/(g*b^5)))^.284/m;
    Yc=fsolve(inline('QG2*(b+2*m*Yc-((b+m*Yc)*Yc)^.333333','Yc','QG2','b','m'),Yc,opt,QG2,b,m);
end
fprintf(1,'Critical Depth =%10.4f\n',Yc);
if WS<Yc
    fprintf(1,'Depth at end of channel is critical\n');  WS=1.05*Yc;
end
NP=20;NP1=NP+1;xup=linspace(0,L,NP1);xdown=L-xup;MOMU=zeros(1,NP1);MOND=zeros(1,NP1);
[xu,Y1ODE]=ode45('fdyxjmp',xup,Yb);
fprintf(1,'Upsteam GVF-profile\n');
fprintf(1,'      x        Y       Mom.     Fr\n');
for i=1:NP1
    YY=Y1ODE(i); A=(b+m*YY)*YY; Fr=sqrt(QG2*(b+m2*YY)/A^3);
    MOMU(i)=(b/2+m*YY/3)*YY^2+QG2/A;
    fprintf(1,'%8.2f %9.3f %10.2f %7.2f\n',xup(i),YY,MOMU(i),Fr);
end
[xd,Y2ODE]=ode45('fdyxjmp',xdown,WS);
fprintf(1,'Downsteam GVF-profile\n');
fprintf(1,'      x        Y       Mom.      Fr\n');
for i=1:NP1
    YY=Y2ODE(i);A=(b+m*YY)*YY; Fr=sqrt(QG2*(b+m2*YY)/A^3);
    MOMD(i)=(b/2+m*YY/3)*YY^2+QG2/A;
    fprintf(1,'%8.2f %9.3f %10.2f %7.2f\n',xdown(i),YY,MOMD(i),Fr);
end
IU=1; NP2=NP+2;
while MOMU(IU)>MOMD(NP2-IU)
    IU=IU+1;
    if IU>=NP1
        disp('Jump does not exist. Flow is supercritical to end of channel');
        return
    end
end
ID=NP2-IU;
xp=(MOMU(IU-1)-MOMD(ID+1))/(MOMU(IU-1)-MOMU(IU)+MOMD(ID)-MOMD(ID+1));
xjump=xup(IU-1)+(xup(IU)-xup(IU-1))*xp;Yups=Y1ODE(IU-1)+xp*(Y1ODE(IU)-Y1ODE(IU-1));
MOM=MOMU(IU-1)+xp*(MOMU(IU)-MOMU(IU-1));Ydow=Y2ODE(ID+1)+xp*(Y2ODE(ID)-Y2ODE(ID+1));
fprintf('Position of Hydraulic jump =%10.2f & Momentum= %10.2f\n',xjump,MOM);
fprintf('Depth upstream =%10.3f, Depth downstream =%10.3f\n',Yups,Ydow);